@@ -9,6 +9,7 @@ from os.path import join
 from re import sub
 from sqlite3 import Connection
 from sqlite3 import Cursor
+from sqlite3 import DatabaseError
 from sqlite3 import connect
 from typing import Generator
 from typing import Optional
@@ -16,6 +17,11 @@ from typing import Union
 
 from chardet import detect as detect_encoding
 from filetype import guess_extension as filetype_guess_extension
+from psutil import AccessDenied
+from psutil import Error
+from psutil import NoSuchProcess
+from psutil import Process
+from psutil import process_iter
 
 from .__version__ import __version__
 from .merge import merge_database
@@ -383,6 +389,35 @@ class FADatabase:
         self.users.reload()
         self.committed_changes: int = self.total_changes
 
+    def check_version(self, raise_for_error: bool = True, *, major: bool = True, minor: bool = True, patch: bool = True
+                      ) -> Optional[DatabaseError]:
+        err: Optional[DatabaseError] = None
+        if (v := self.version) == __version__:
+            return None
+        if not v:
+            err = DatabaseError(f"version error: {v}")
+        if (v_db := v.split("."))[0] != (v_md := __version__.split("."))[0]:
+            err = DatabaseError(f"major version is not latest: {v_db[0]} != {v_md[0]}") if major else None
+        elif v_db[1] != v_md[1]:
+            err = DatabaseError(f"minor version is not latest: {v_db[1]} != {v_md[1]}") if minor else None
+        elif v_db[2] != v_md[2]:
+            err = DatabaseError(f"patch version is not latest: {v_db[2]} != {v_md[2]}") if patch else None
+        if err is not None and raise_for_error:
+            raise err
+        return err
+
+    def check_connection(self, raise_for_error: bool = True) -> list[Process]:
+        ps: list[Process] = []
+        for process in process_iter():
+            try:
+                if process.is_running() and any(self.database_path == path for path, _fd in process.open_files()):
+                    ps.append(process)
+            except (NoSuchProcess, AccessDenied):
+                pass
+        if len(ps) > 1 and raise_for_error:
+            raise Error(f"Multiple connections to database: {ps}")
+        return ps
+
     def commit(self):
         self.connection.commit()
         self.committed_changes = self.total_changes
